#include <LedControl.h>

// Pin configuration for the LED matrix
int DIN = 7;   // Data-in
int CLK = 6;   // Clock
int CS = 5;    // Chip Select

// Create a LedControl object
LedControl lc = LedControl(DIN, CLK, CS, 1);

// Smoke sensor pin
const int smokeSensor = A0;

// Buzzer pin
const int buzzer = 3;

// Threshold value for smoke detection
const int smokeThreshold = 400;

// Define the bitmap for a smiley face
byte smileyFace[8] = {
  0b00111100,
  0b01000010,
  0b10100101,
  0b10000001,
  0b10100101,
  0b10011001,
  0b01000010,
  0b00111100
};

// Define the bitmap for a sad face
byte sadFace[8] = {
  0b00111100,
  0b01000010,
  0b10100101,
  0b10000001,
  0b10011001,
  0b10100101,
  0b01000010,
  0b00111100
};

void setup() {
  // Initialize the LED matrix
  lc.shutdown(0, false);       // Wake up displays
  lc.setIntensity(0, 8);       // Set brightness level (0 is min, 15 is max)
  lc.clearDisplay(0);          // Clear display register

  // Initialize the smoke sensor and buzzer pin as input and output, respectively
  pinMode(smokeSensor, INPUT);
  pinMode(buzzer, OUTPUT);

  // Start the serial communication
  Serial.begin(9600);
}

void loop() {
  // Read the analog value from the smoke sensor
  int analogValue = analogRead(smokeSensor);

  // If the analog value is above the threshold, smoke is detected
  if (analogValue > smokeThreshold) {
    // Display a sad face on the matrix
    displayFace(sadFace);
    Serial.println("Smoke detected! :(");

    // Beep the buzzer
    tone(buzzer, 1000); // Set buzzer to 1kHz
    delay(400);         // Leave the buzzer on for 400ms
    noTone(buzzer);     // Turn the buzzer off
    delay(400);         // Pause for 400ms
  } else {
    // Display a smiley face on the matrix
    displayFace(smileyFace);
    Serial.println("Air is clear. :)");
    noTone(buzzer);     // Ensure the buzzer is off
  }

  // Delay a bit before reading the sensor again
  delay(1000);
}

void displayFace(byte face[]) {
  for (int row = 0; row < 8; row++) {
    lc.setRow(0, row, face[row]);
  }
}
